@@ -149,7 +149,7 @@ hcEII <- function(data, partition, minclus = 1, ...)
     stop("missing values not allowed in data")
   #====================================================================
   dimdat <- dim(data)
-  oneD <- (is.null(dimdat) || length(dimdat[dimdat > 1]) == 1)
+  oneD <- (is.null(dimdat) || NCOL(data) == 1)
   #if(oneD || length(dimdat) > 2)
   #  stop("data should in the form of a matrix")
   data <- as.matrix(data)
@@ -196,7 +196,7 @@ hcEEE <- function(data, partition, minclus = 1, ...)
     stop("missing values not allowed in data")
   #=====================================================================
   dimdat <- dim(data)
-  oneD <- (is.null(dimdat) || length(dimdat[dimdat > 1]) == 1)
+  oneD <- (is.null(dimdat) || NCOL(data) == 1)
   #if(oneD || length(dimdat) > 2)
   #  stop("data should in the form of a matrix")
   data <- as.matrix(data)
@@ -262,7 +262,7 @@ hcVII <- function(data, partition, minclus = 1, alpha = 1, ...)
     stop("missing values not allowed in data")
   #=====================================================================
   dimdat <- dim(data)
-  oneD <- (is.null(dimdat) || length(dimdat[dimdat > 1]) == 1)
+  oneD <- (is.null(dimdat) || NCOL(data) == 1)
   #if(oneD || length(dimdat) > 2)
   #  stop("data should in the form of a matrix")
   data <- as.matrix(data)
@@ -311,7 +311,7 @@ hcVVV <- function(data, partition, minclus = 1, alpha = 1, beta = 1, ...)
   if(any(is.na(data)))
     stop("missing values not allowed in data")
   dimdat <- dim(data)
-  oneD <- (is.null(dimdat) || length(dimdat[dimdat > 1]) == 1)
+  oneD <- (is.null(dimdat) || NCOL(data) == 1)
   #if(oneD || length(dimdat) > 2)
   #  stop("data should in the form of a matrix")
   data <- as.matrix(data)
@@ -408,7 +408,7 @@ hcE <- function(data, partition, minclus = 1, ...)
     stop("missing values not allowed in data")
   #====================================================================
   dimdat <- dim(data)
-  oneD <- is.null(dimdat) || length(dimdat[dimdat > 1]) == 1
+  oneD <- (is.null(dimdat) || NCOL(data) == 1)
   if(!oneD)
     stop("data must be one-dimensional")
   data <- as.vector(data)
@@ -449,7 +449,7 @@ hcV <- function(data, partition, minclus = 1, alpha = 1, ...)
     stop("missing values not allowed in data")
   #=====================================================================
   dimdat <- dim(data)
-  oneD <- is.null(dimdat) || length(dimdat[dimdat > 1]) == 1
+  oneD <- (is.null(dimdat) || NCOL(data) == 1)
   if(!oneD)
     stop("data must be one-dimensional")
   data <- as.vector(data)
